%  Chi-Li Chiu 20180823
%
%  
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Surface menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Surfaces Functions">
%        <Item name="Surface Intensity Hisrogram" icon="Matlab">
%          <Command>MatlabXT::XT_Intensity_Histogram(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="Surface Intensity Histogram" icon="Matlab">
%            <Command>MatlabXT::XT_Intensity_Histogram(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%  Generates intensity histogram for each channel and each time point within selected Surface Object
%  Allows histogram export to .csv format.
 

function XT_Intensity_Histogram(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end


% the user has to create a scene with some surfaces
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
  msgbox('Please create some Surfaces in the Surpass scene!');
  return;
end

% get the surfaces
vSurfaces = vImarisApplication.GetFactory.ToSurfaces(vImarisApplication.GetSurpassSelection);

% search the surfaces if not previously selected
if ~vImarisApplication.GetFactory.IsSurfaces(vSurfaces)        
  for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
    vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
    if isequal(vSurfaces, [])
      if vImarisApplication.GetFactory.IsSurfaces(vDataItem)
        vSurfaces = vImarisApplication.GetFactory.ToSurfaces(vDataItem);
      end
    end
  end
  % did we find the surfaces?
  if isequal(vSurfaces, [])
    msgbox('Please create some surfaces!');
    return;
  end
end

%get dataset info
vDataset = vImarisApplication.GetDataSet;
vSizeX = vDataset.GetSizeX;
vSizeY = vDataset.GetSizeY;
vSizeZ = vDataset.GetSizeZ;
vSizeC = vDataset.GetSizeC;
vSizeT = vDataset.GetSizeT;
vMaxX = vDataset.GetExtendMaxX;
vMaxY = vDataset.GetExtendMaxY;
vMaxZ = vDataset.GetExtendMaxZ;
vMinX = vDataset.GetExtendMinX;
vMinY = vDataset.GetExtendMinY;
vMinZ = vDataset.GetExtendMinZ;

SurfaceIntTotal = zeros(vSizeC,vSizeT,vSizeX*vSizeY*vSizeZ);

%Surface Mask
for vIndexC = 0:vSizeC-1
    for vIndexT = 0:vSizeT-1
        vMask = vSurfaces.GetMask(vMinX,vMinY,vMinZ,vMaxX,vMaxY,vMaxZ,vSizeX,vSizeY,vSizeZ,vIndexT);
        vMaskVolume = vMask.GetDataVolumeAs1DArrayFloats(0,0);
        vMaskVolume(vMaskVolume == 0) = NaN;
        vData = vDataset.GetDataVolumeAs1DArrayFloats(vIndexC,vIndexT);
        SurfaceInt = vMaskVolume.*vData;
        SurfaceIntTotal(vIndexC+1,vIndexT+1,:) = SurfaceInt;
    end
end

minBin = min(SurfaceIntTotal(:));
maxBin = max(SurfaceIntTotal(:));

%dialog to specify bin max, bin min, number of bins
prompt = {'Min bin edge:','Max bin edge:','Total bin number:'};
title = 'Intensity histogram';
dims = [1 35];
definput = {num2str(minBin),num2str(maxBin),'40'};
hInput = inputdlg(prompt,title,dims,definput);

minBin = str2double(hInput(1));
maxBin = str2double(hInput(2));
nBins = str2double(hInput(3));

%calculate histcounts for all channels and time points
bincountsTotal = zeros(nBins,vSizeC,vSizeT);
edges = linspace(minBin,maxBin,nBins+1);
for vIndexC = 1:vSizeC
    for vIndexT = 1:vSizeT
        bincountsTotal(:,vIndexC,vIndexT) = histcounts(SurfaceIntTotal(vIndexC,vIndexT,:),edges);
    end
end

for vIndexC = 1:vSizeC
    f = figure;
    set(f,'Name',strcat('Ch',num2str(vIndexC),' time point1'));
    set(gcf,'Color',[1,1,1]);
    ax = axes('Parent',f,'position',[0.13 0.39  0.77 0.54]);
    h = histogram('BinEdges',edges,'BinCounts',squeeze(bincountsTotal(:,vIndexC,1)));
    xlim([minBin maxBin]);
    h.FaceColor = 'k';
    if vSizeT > 1
        uicontrol('Parent',f,'Style','slider','Position',[81,54,419,23],...
                  'value',1, 'min',1, 'max',vSizeT,'Callback',{@timeUpdate,bincountsTotal,vIndexC,f,minBin,maxBin,nBins});
    end
    
    %export to csv
    hTotal = zeros(nBins+2,vSizeT+1);        
    hTotal(2:end,1) = edges';
    for vIndexT = 1:vSizeT
        hTotal(2:end-1,vIndexT+1) = squeeze(bincountsTotal(:,vIndexC,vIndexT));
    end
    
    %add csv header
    TimePoints = string(1:vSizeT);
    Header = [TimePoints];
    hTotal(1,2:end) = Header;
    hTotal(1,1) = nan;
    
    btn = uicontrol('Style', 'pushbutton', 'String', 'Export this channel',...
        'Position', [20 20 100 20],...
        'Callback', {@dataExport,hTotal});  
    
end

function dataExport(hObj,~,hTotal)
filter = {'*.csv'};
[file, path] = uiputfile(filter,'Save histogram');
filename = fullfile(path,file);
if isequal(file,0) || isequal(path,0)
   disp('Canceled');
else
   csvwrite(filename,hTotal); 
end

function timeUpdate(hObj,~,bincountsTotal,vIndexC,f,minBin,maxBin,nBins)
timePoint = int8(hObj.Value);
edges = linspace(minBin,maxBin,nBins+1);
bincounts = squeeze(bincountsTotal(:,vIndexC,timePoint));
h = histogram('BinEdges',edges,'BinCounts',bincounts);
xlim([minBin maxBin]);
    
h.FaceColor = 'k';
set(f,'Name',strcat('Ch',num2str(vIndexC),' time point',num2str(timePoint)));